import functools
import os
import time

from locust import events,User

from xlocust_bigquery.bigquery import query, load_data_gs_bucket_csv, load_data_gs_bucket_json
from google.cloud import bigquery
from google.cloud.bigquery.table import RowIterator, _EmptyRowIterator
from google.oauth2 import service_account


class BQClientUser(User):

    abstract = True

    def __init__(self, environment):
        super().__init__(environment)

    def client(self):
        account = os.environ.get("GOOGLE_APPLICATION_CREDENTIALS")

        credentials = service_account.Credentials.from_service_account_file(account, scopes=[
            "https://www.googleapis.com/auth/cloud-platform"])

        self.bqclient = bigquery.Client(credentials=credentials, project=credentials.project_id)
    
    def query(self,query):

        start_time = time.time()

        try:
            bqclient = self.client()
            bq_query = bqclient.query(query)
            results = bq_query.result()
            data = results
            return data
        except Exception as e:
            self.environment.events.request.fire(
            request_type="GCPBQ",
            name=query,
            response_time=int((time.time() - start_time) * 1000),
            response_length=int((time.time() - start_time) * 1000),
            context={},
            exception=e,
    )



    # def dataload_csv_gsbucket(self,environment, uri, table_id,response_length,start_time,err, _msg):
        
    #     self.environment.events.request.fire(
    #     request_type="GCPBQ",
    #     name=uri+"_"+table_id,
    #     response_time=(time.monotonic() - start_time) * 1000,
    #     response_length=response_length,
    #     context={},
    #     exception=err,
    # )


    


import os
from typing import Union

from dotenv import load_dotenv, find_dotenv
from google.cloud import bigquery
from google.cloud.bigquery.table import RowIterator, _EmptyRowIterator
from google.oauth2 import service_account

load_dotenv(find_dotenv())


def client():
    account = os.environ.get("GOOGLE_APPLICATION_CREDENTIALS")

    credentials = service_account.Credentials.from_service_account_file(account, scopes=[
        "https://www.googleapis.com/auth/cloud-platform"])

    bqclient = bigquery.Client(credentials=credentials, project=credentials.project_id)

    return bqclient


def query(query):
    bqclient = client()
    bq_query = bqclient.query(query)
    results: Union[RowIterator, _EmptyRowIterator] = bq_query.result()
    data = results
    return data


def load_data_gs_bucket_csv(uri, table_id):
    bqclient = client()
    job_config = bigquery.LoadJobConfig(
        autodetect=False,
        skip_leading_rows=1,
        source_format=bigquery.SourceFormat.CSV
    )
    job_config.schema_update_options = [
        bigquery.SchemaUpdateOption.ALLOW_FIELD_RELAXATION
    ]
    load_job = bqclient.load_table_from_uri(uri, table_id, job_config=job_config)

    load_job.result()

    destination_table = bqclient.get_table(table_id)
    print("Loaded {} rows.".format(destination_table.num_rows))
    return destination_table.num_rows


def load_data_gs_bucket_json(uri, table_id):
    bqclient = client()
    job_config = bigquery.LoadJobConfig(
        autodetect=True,
        skip_leading_rows=1,
        source_format=bigquery.SourceFormat.NEWLINE_DELIMITED_JSON
    )
    job_config.schema_update_options = [
        bigquery.SchemaUpdateOption.ALLOW_FIELD_RELAXATION,
        bigquery.SchemaUpdateOption.ALLOW_FIELD_ADDITION
    ]
    load_job = bqclient.load_table_from_uri(uri, table_id, job_config=job_config)

    load_job.result()

    destination_table = bqclient.get_table(table_id)
    print("Loaded {} rows.".format(destination_table.num_rows))



from setuptools import setup, find_packages

setup(
    name='xlocust_bigquery',
    version='0.0.2',
    packages=find_packages(),
    url='https://github.com/KelvinSan/xlocust_bigquery',
    license='',
    author='kelvin sanyaolu',
    author_email='kelvin.sanyaolu@gmail.com',
    description='A Google cloud GP Client for locustio to test dataloads and query extraction'
)
